@@ -1,8 +1,4 @@
-import threading
-from logging import basicConfig, INFO, getLogger
-
-logger = getLogger(__name__)
-basicConfig(level=INFO, format='[%(levelname)s]: %(asctime)s %(message)s', datefmt='%Y-%m-%d %H:%M:%S')
+from executor.log_support import logger
 
 
 def insert_log(function: str, table: str, **kwargs):
