@@ -12,3 +12,8 @@ print(isinstance(s, Collection), isinstance(s, Sequence))
 s = 'hello'
 print(isinstance(s, Collection), isinstance(s, Sequence))
 # True True
+
+import executor
+from config import MYSQL
+
+executor.init_db(**MYSQL)
